self.__BUILD_MANIFEST=function(s,c,e,a,t,u,n,f,i,b,o,j,r,k,d){return{__rewrites:{afterFiles:[],beforeFiles:[],fallback:[]},"/":["static/chunks/pages/index-444f1804401f04ea.js"],"/_error":["static/chunks/pages/_error-c66a4e8afc46f17b.js"],"/clusters":["static/chunks/pages/clusters-9e5d47818b9bdadd.js"],"/clusters/[cluster]":[c,s,e,u,n,o,b,a,t,f,j,i,r,k,d,"static/chunks/6856-e02fb3cb28d0d887.js","static/chunks/1871-7e202677c42f43fe.js","static/chunks/pages/clusters/[cluster]-a7565f586ef86467.js"],"/clusters/[cluster]/[job]":[c,s,e,u,a,t,i,"static/chunks/pages/clusters/[cluster]/[job]-33f525539665fdfd.js"],"/config":["static/chunks/pages/config-718cdc365de82689.js"],"/infra":["static/chunks/pages/infra-d187cd0413d72475.js"],"/infra/[context]":["static/chunks/pages/infra/[context]-12c559ec4d81fdbd.js"],"/jobs":["static/chunks/pages/jobs-e5a98f17f8513a96.js"],"/jobs/pools/[pool]":[c,s,e,n,b,a,t,f,"static/chunks/pages/jobs/pools/[pool]-8d0f4655400b4eb9.js"],"/jobs/[job]":[c,s,e,u,n,b,a,t,f,i,"static/chunks/pages/jobs/[job]-895847b6cf200b04.js"],"/plugins/[...slug]":[s,"static/chunks/pages/plugins/[...slug]-4f46050ca065d8f8.js"],"/users":["static/chunks/pages/users-2f7646eb77785a2c.js"],"/volumes":["static/chunks/pages/volumes-ef19d49c6d0e8500.js"],"/workspace/new":["static/chunks/pages/workspace/new-3f88a1c7e86a3f86.js"],"/workspaces":["static/chunks/pages/workspaces-cb4da3abe08ebf19.js"],"/workspaces/[name]":[c,s,e,u,n,o,a,t,f,j,i,r,k,d,"static/chunks/1141-9c810f01ff4f398a.js","static/chunks/pages/workspaces/[name]-96e0f298308da7e2.js"],sortedPages:["/","/_app","/_error","/clusters","/clusters/[cluster]","/clusters/[cluster]/[job]","/config","/infra","/infra/[context]","/jobs","/jobs/pools/[pool]","/jobs/[job]","/plugins/[...slug]","/users","/volumes","/workspace/new","/workspaces","/workspaces/[name]"]}}("static/chunks/5739-d67458fcb1386c92.js","static/chunks/616-3d59f75e2ccf9321.js","static/chunks/6130-2be46d70a38f1e82.js","static/chunks/6989-01359c57e018caa4.js","static/chunks/3850-fd5696f3bbbaddae.js","static/chunks/7411-b15471acd2cba716.js","static/chunks/1272-1ef0bf0237faccdb.js","static/chunks/8969-452f9d5cbdd2dc73.js","static/chunks/9353-8369df1cf105221c.js","static/chunks/6212-7bd06f60ba693125.js","static/chunks/7359-c8d04e06886000b3.js","static/chunks/6990-09cbf02d3cd518c3.js","static/chunks/2260-7703229c33c5ebd5.js","static/chunks/3800-b589397dc09c5b4e.js","static/chunks/7615-019513abc55b3b47.js"),self.__BUILD_MANIFEST_CB&&self.__BUILD_MANIFEST_CB();